#
# Author: Ha Ho-Hagemann (Hereon, Germany)
# Contact: ha.hagemann@hereon.de
#
#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
#
created_date="15-Jul-2022 09:50"
#
#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
WRKDIR=$(pwd)

runstate=$1

if [ -e ./job_settings ] ; then 
  source ./job_settings
else
  echo 'Script job_settings does not exist in present directory scr!'
  exit
fi

if [ -e ./setuserdir.com ] ; then
  source ./setuserdir.com
else
  echo 'Script setuserdir.com does not exist in present directory scr!'
  exit
fi

#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
#
# (1) Fill model list
# -------------------
#
export EXPNAME=${EXP}

modelname[0]=${MODEL1[0]}
modelcomponent[0]=${MODEL1[1]}
modelgrid[0]=${MODEL1[2]}
model_id[0]=${MODEL1[3]}

modelname[1]=${MODEL2[0]}
modelcomponent[1]=${MODEL2[1]}
modelgrid[1]=${MODEL2[2]}
model_id[1]=${MODEL2[3]}

modelname[2]=${MODEL3[0]}
modelcomponent[2]=${MODEL3[1]}
modelgrid[2]=${MODEL3[2]}
model_id[2]=${MODEL3[3]}

tran_id[0]=${exchanged_var1[0]} # "1"
tran_id[1]=${exchanged_var2[0]} # "2"
tran_id[2]=${exchanged_var3[0]} # "3"

var[0]=${exchanged_var1[1]} 	# "RDC2NEMO"
var[1]=${exchanged_var2[1]} 	# "RUNOFF_S"
var[2]=${exchanged_var3[1]} 	# "RUNOFF_G"

collsize[0]=${exchanged_var1[2]} # "1"
collsize[1]=${exchanged_var2[2]} # "2"
collsize[2]=${exchanged_var3[2]} # "3"

# (2) start and end date+time of experiment
# -------------------------------------
if [ ${runstate} == "restart" ];then
  YYYY=`cat ${WRKDIR}/log/${EXP}.year`
fi
YYYY_NEXT=$((${YYYY}+1))

start_date=${start_date:="${YYYY}-01-01T00:00:00"}
#    end_date=${end_date:="${YYYY_NEXT}-01-01T00:00:00"}
    end_date=${end_date:="${YYYY}-12-31T00:00:00"}

#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# (3) prepare coupling.xml
#
# *** 3.1: interpolation_method_file
#
#------ source_to_target_map.txt -------------------
interpolation_method_file="source_to_target_map.txt"
cat > ${interpolation_method_file} << EOF
                                <interpolation_requirements>
                                        <interpolation method="source_to_target_map"
                                                       max_search_distance="3.6"/>
                                        <interpolation method="fixed_value" user_value="0.0"/>
                                </interpolation_requirements>
EOF

#------ bernstein_bezier.txt -------------------
interpolation_method_file="bernstein_bezier.txt"
cat > ${interpolation_method_file} << EOF
            <mapping_on_source>true</mapping_on_source>
            <interpolation_requirements>
               <interpolation method="bernstein_bezier"/>
               <interpolation gauss_scale="0.1" method="n-nearest_neighbor" n="4" weighted="ARITHMETIC_AVERAGE"/>
               <interpolation method="fixed_value" user_value="-999.9"/>
            </interpolation_requirements>
            <enforce_write_weight_file filename="">false</enforce_write_weight_file>
EOF

#------ conservative.txt -------------------
interpolation_method_file="conservative.txt"
cat > ${interpolation_method_file} << EOF
            <mapping_on_source>true</mapping_on_source>
            <interpolation_requirements>
               <interpolation enforced_conservation="false" method="conservative" normalisation="FRACAREA" order="1" partial_coverage="true"/>
               <interpolation method="fixed_value" user_value="-999.9"/>
            </interpolation_requirements>
            <enforce_write_weight_file filename="">false</enforce_write_weight_file>
EOF

#------ user_file.txt -------------------
interpolation_method_file="user_file.txt"
cat > ${interpolation_method_file} << EOF
                                <interpolation_requirements>
                                        <interpolation method="user_file"
                                        	       filename="${var[0]}.nc"/>
                                </interpolation_requirements>
EOF
#
# *** 3.2: coupling.xml file
#
# component names in coupling.xml must (!) match with modelname[*]
#
coupling_xml_file="coupling_${EXPNAME}.xml"

cat > ${coupling_xml_file} << EOF
<?xml version="1.0" encoding="UTF-8"?>

<!-- Example XML document -->
<coupling
        xmlns="http://www.w3schools.com"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://www.w3schools.com coupling.xsd">

        <redirect redirect_of_root="false" redirect_stdout="false"/>

        <components>
                <!--ocean-->
                <component id="${model_id[0]}"> 
                        <name>${modelname[0]}</name>
                        <model>${modelname[0]}</model>
                        <simulated>${modelcomponent[0]}</simulated>
                        <transient_grid_refs>
                                <!--RDC2NEMO-->
                                <transient_grid_ref id="1" transient_ref="${tran_id[0]}" grid_ref="${model_id[0]}" collection_size="${collsize[0]}"/>
                        </transient_grid_refs>
                </component>

                <!--land-->
                <component id="${model_id[1]}">
                        <name>${modelname[1]}</name>
                        <model>${modelname[1]}</model>
                        <simulated>${modelcomponent[1]}</simulated>
                        <transient_grid_refs>
                                <!--RUNOFF_S-->
                                <transient_grid_ref id="1" transient_ref="${tran_id[1]}" grid_ref="${model_id[1]}" collection_size="${collsize[1]}"/>
                                <!--RUNOFF_S-->
                                <transient_grid_ref id="2" transient_ref="${tran_id[2]}" grid_ref="${model_id[1]}" collection_size="${collsize[2]}"/>
                        </transient_grid_refs>
                </component>

                <!--HD-->
                <component id="${model_id[2]}">
                        <name>${modelname[2]}</name>
                        <model>${modelname[2]}</model>
                        <simulated>${modelcomponent[2]}</simulated>
                        <transient_grid_refs>
                                <!--RDC2NEMO-->
                                <transient_grid_ref id="1" transient_ref="${tran_id[0]}" grid_ref="${model_id[2]}" collection_size="${collsize[0]}"/>
                                <!--RUNOFF_S-->
                                <transient_grid_ref id="2" transient_ref="${tran_id[1]}" grid_ref="${model_id[2]}" collection_size="${collsize[1]}"/>
                                <!--RUNOFF_G-->
                                <transient_grid_ref id="3" transient_ref="${tran_id[2]}" grid_ref="${model_id[2]}" collection_size="${collsize[2]}"/>
                        </transient_grid_refs>
                </component>
        </components>

        <transients>
                <transient id="${tran_id[0]}" transient_standard_name="${var[0]}"/>
                <transient id="${tran_id[1]}" transient_standard_name="${var[1]}"/>
                <transient id="${tran_id[2]}" transient_standard_name="${var[2]}"/>
        </transients>

        <grids>
                <grid id="${model_id[0]}" alias_name="${modelgrid[0]}"/>
                <grid id="${model_id[1]}" alias_name="${modelgrid[1]}"/>
                <grid id="${model_id[2]}" alias_name="${modelgrid[2]}"/>
        </grids>

        <dates>
                <start_date>${start_date}</start_date>
                <end_date>${end_date}</end_date>
                <calendar>proleptic-gregorian</calendar>
        </dates>
        <timestep_unit>second</timestep_unit>

        <couples>
                <couple>
                        <!--ocean-->
                        <component1 component_id="1"/>
                        <!--HD-->
                        <component2 component_id="3"/>

                        <!--RDC2NEMO-->
                        <transient_couple transient_id="${tran_id[0]}">
                                <source component_ref="2" transient_grid_ref="1"/>
                                <target                   transient_grid_ref="1"/>
                                <timestep>
                                        <source>${rivTimeStep}</source>
                                        <target>${oceTimeStep}</target>
                                        <coupling_period operation="accumulate">${couplingTimeStep}</coupling_period>
                                        <source_timelag>${riv_lag}</source_timelag>
                                        <target_timelag>${oce_lag}</target_timelag>
                                </timestep>
EOF
if [ ${runstate} == "restart" ];then
  cat user_file.txt >> ${coupling_xml_file}
else
  cat source_to_target_map.txt >> ${coupling_xml_file}
cat >> ${coupling_xml_file} << EOF
                                <enforce_write_weight_file filename="${var[0]}.nc">true</enforce_write_weight_file>
EOF
fi
cat >> ${coupling_xml_file} << EOF
                        </transient_couple>

                </couple>

                <couple>
                        <!--land-->
                        <component1 component_id="2"/>
                        <!--HD-->
                        <component2 component_id="3"/>

                        <!--RUNOFF_S-->
                        <transient_couple transient_id="${tran_id[1]}">
                                <source component_ref="1" transient_grid_ref="1"/>
                                <target                   transient_grid_ref="2"/>
                                <timestep>
                                        <source>${lndTimeStep}</source>
                                        <target>${rivTimeStep}</target>
                                        <coupling_period operation="accumulate">${couplingTimeStep}</coupling_period>
                                        <source_timelag>${lnd_lag}</source_timelag>
                                        <target_timelag>${riv_lag}</target_timelag>
                                </timestep>
EOF
if [ ${runstate} == "restart" ];then
  cat user_file.txt >> ${coupling_xml_file}
else
  cat source_to_target_map.txt >> ${coupling_xml_file}
cat >> ${coupling_xml_file} << EOF
                                <enforce_write_weight_file filename="${var[1]}.nc">true</enforce_write_weight_file>
EOF
fi
cat >> ${coupling_xml_file} << EOF
                        </transient_couple>

                        <!--RUNOFF_G-->
                        <transient_couple transient_id="${tran_id[2]}">
                                <source component_ref="1" transient_grid_ref="2"/>
                                <target                   transient_grid_ref="3"/>
                                <timestep>
                                        <source>${lndTimeStep}</source>
                                        <target>${oceTimeStep}</target>
                                        <coupling_period operation="accumulate">${couplingTimeStep}</coupling_period>
                                        <source_timelag>${lnd_lag}</source_timelag>
                                        <target_timelag>${riv_lag}</target_timelag>
                                </timestep>
EOF
if [ ${runstate} == "restart" ];then
  cat user_file.txt >> ${coupling_xml_file}
else
  cat source_to_target_map.txt >> ${coupling_xml_file}
cat >> ${coupling_xml_file} << EOF
                                <enforce_write_weight_file filename="${var[2]}.nc">true</enforce_write_weight_file>
EOF
fi
cat >> ${coupling_xml_file} << EOF
                        </transient_couple>
                        
                </couple>
        </couples>
   <created date="${created_date}"/>
</coupling>
EOF

